/*
 *	fonts.c
 */

#include "filer.h"

#define MAX_FONTS	12

typedef struct _FONTS {
	u8		fontname[XF_NAMELEN+1];
	int		fontsize;
	PXFONTHEAD	pxa;
	PXFONTHEAD	pxk;
} FONTS, *PFONTS;

typedef struct _FONTPARAMS {
	u8		fontname[XF_NAMELEN+1];
	int		fontsize;
} FONTPARAMS, *PFONTPARAMS;

static const u8	FontPath[] = "\\system\\fonts";

static FONTS	aFonts[MAX_FONTS];
static int	nFonts;

static int	CurFontIndex;

/*
 *	IsValidFontSize
 */
static BOOL IsValidFontSize(PFONTSIZE pfs)
{
	if (memcmp((u8 *)pfs->sig, FS_SIG, sizeof(pfs->sig)) ||
		pfs->ver != FS_VER ||
		pfs->size != sizeof(*pfs))
		return FALSE;
	return TRUE;
}

/*
 *	LoadFont
 */
static void LoadFont(PFONTPARAMS pfp)
{
	PFONTSIZE	pfs;

	pfs = (PFONTSIZE)FONTSIZE_ADDR;
	if (!IsValidFontSize(pfs)) {
		memset(pfp->fontname, 0, sizeof(pfp->fontname));
		pfp->fontsize = 0;
		return;
	}

	ReadSRAMByte(pfs->fontname, pfp->fontname, sizeof(pfp->fontname));
	pfp->fontsize = pfs->fontsize;
}

/*
 *	SaveFont
 */
static void SaveFont(PFONTPARAMS pfp)
{
	PFONTSIZE	pfs;

	pfs = (PFONTSIZE)FONTSIZE_ADDR;
	if (!IsValidFontSize(pfs)) {
		/*
		 *	Build fontsize
		 */
		WriteSRAMByte(pfs->sig, FS_SIG, sizeof(pfs->sig));
		WriteSRAM8(&pfs->ver, FS_VER);
		WriteSRAM8(&pfs->size, sizeof(*pfs));
	}

	WriteSRAMByte(pfs->fontname, pfp->fontname, sizeof(pfs->fontname));
	WriteSRAM8(&pfs->fontsize, pfp->fontsize);
}

/*
 *	FindFont
 */
static int FindFont(PFONTPARAMS pfp)
{
	int		i;
	PFONTS		pf;

	for (i = 0, pf = aFonts; i < nFonts; i++, pf++) {
		if (strcmp(pf->fontname, pfp->fontname) == 0 &&
			pf->fontsize == pfp->fontsize)
			break;
	}

	return i;
}

/*
 *	InitFonts
 */
void InitFonts()
{
	PDIRENT		pdir;
	PDIRENT		pd, pe;
	PXFONTHEAD	pxf;
	PFONTS		pf, pf2;
	FONTS		ftmp;
	int		i, j;
	FONTPARAMS	parms;

	if (!(pdir = FsGetDirEnt((u8 *)FontPath))) {
		Fix8PutStr(10, 80, "InitFonts: Can't find font directory.");
		for (;;) ;
	}

	/*
	 *	Get all fonts
	 */

	nFonts = 0;
	pd = (PDIRENT)(pdir->fileaddr + pdir->namesize);
	pe = (PDIRENT)((u8 *)pd + pdir->filesize);
	for (; pd < pe; pd++) {
		if (pd->attr & DA_DIR)
			continue;
		pxf = (PXFONTHEAD)(pd->fileaddr + pd->namesize);
		if (!IsValidXfont(pxf))
			continue;

		for (i = 0, pf = aFonts; i < nFonts; i++, pf++) {
			if (strcmp(pf->fontname, pxf->name) == 0 &&
				pf->fontsize == pxf->height)
				break;
		}
		if (i >= MAX_FONTS)
			break;
		if (i >= nFonts) {
			memcpy(pf->fontname, pxf->name, sizeof(pf->fontname));
			pf->fontsize = pxf->height;
			nFonts++;
		}

		if (ISDBCSFONT(pxf))
			pf->pxk = pxf;
		else
			pf->pxa = pxf;
	}

	/*
	 *	Check fonts
	 */

	if (nFonts <= 0) {
		Fix8PutStr(10, 80, "InitFonts: fonts not found.");
		for (;;) ;
	}

	for (i = 0, pf = aFonts; i < nFonts - 1; i++, pf++)
		if (!pf->pxa) {
			Fix8PutStr(10, 80, "InitFonts: No alphabet font");
			for (;;) ;
		}

	/*
	 *	Sort fonts
	 */

	for (i = 0, pf = aFonts; i < nFonts - 1; i++, pf++)
		for (j = i + 1, pf2 = pf + 1; j < nFonts; j++, pf2++)
			if (pf->fontsize > pf2->fontsize ||
				pf->pxk->width > pf2->pxk->width) {
				ftmp = *pf;
				*pf = *pf2;
				*pf2 = ftmp;
			}

	/*
	 *	Load default font size and set the font
	 */

	pf = &aFonts[0];	// Smallest font
	XfontSetFontObject(FO_SMALL, pf->pxa, pf->pxk);

	LoadFont(&parms);
	CurFontIndex = FindFont(&parms);
	if (CurFontIndex >= nFonts)
		CurFontIndex = 0;

	pf = &aFonts[CurFontIndex];
	XfontSetFontObject(FO_NORMAL, pf->pxa, pf->pxk);

	XfontSelectFontObject(FO_NORMAL);
	XfontSetColor(COLOR_WHITE, COLOR_BLACK);
}

/*
 *	SelectFont
 */
BOOL SelectFont()
{
	int		i;
	PFONTS		pf;
	LISTBOX		lb;
	FONTPARAMS	parms;
	u8		*pitems[MAX_FONTS];
	u8		items[MAX_FONTS][16];

	/*
	 *	Choose font
	 */

	lb.title = "Select Font";
	lb.nitems = nFonts;
	lb.defitem = CurFontIndex;
	for (i = 0; i < nFonts; i++) {
		pitems[i] = &items[i][0];
		_sprintf(pitems[i], "%s%3d",
			aFonts[i].fontname, aFonts[i].fontsize);
	}
	lb.items = pitems;

	i = ListBox(&lb);

	if (i < 0 || i == CurFontIndex)
		return FALSE;

	/*
	 *	Select font
	 */

	CurFontIndex = i;
	pf = &aFonts[CurFontIndex];

	XfontSetFontObject(FO_NORMAL, pf->pxa, pf->pxk);

	memcpy(parms.fontname, pf->fontname, sizeof(parms.fontname));
	parms.fontsize = pf->fontsize;
	SaveFont(&parms);

	return TRUE;
}

