/*
 *	subr.c
 */

#include <stdarg.h>
#include "filer.h"

#define VRAM_END	((vu16 *)VRAM_ADDR + SCREEN_WIDTH * SCREEN_HEIGHT)

/*
 *	ClearScreen
 */
void ClearScreen()
{
	vu16		*vp;

	vp = (vu16 *)VRAM_ADDR; 
	while (vp < VRAM_END)
		*vp++ = 0;
}

/*
 *	FillScreen
 */
void FillScreen(int x, int y, int width, int height, u16 c)
{
	vu16		*vp0, *vp;
	int		i;

	vp0 = (vu16 *)VRAM_ADDR + y * SCREEN_WIDTH + x;
	while (height-- > 0) {
		vp = vp0;
		for (i = 0; i < width; i++)
			*vp++ = c;
		vp0 += SCREEN_WIDTH;
	}
}

/*
 *	EraseScreen
 */
void EraseScreen(int x, int y, int width, int height)
{
	FillScreen(x, y, width, height, COLOR_BLACK);
}

/*
 *	ReverseScreen
 */
void ReverseScreen(int x, int y, int width, int height)
{
	vu16		*vp0, *vp;
	int		i;

	vp0 = (vu16 *)VRAM_ADDR + y * SCREEN_WIDTH + x;
	while (height-- > 0) {
		vp = vp0;
		for (i = 0; i < width; i++)
			*vp++ ^= 0x7fff;
		vp0 += SCREEN_WIDTH;
	}
}

/*
 *	CopyBitmap16
 */
void CopyBitmap16(PBITMAP pbm, int x, int y)
{
	vu16		*pv;
	const u16	*pi;
	int		w, h;

	pv = (vu16 *)VRAM_ADDR + (y * SCREEN_WIDTH) + x;
	pi = pbm->pimage;
	h = pbm->height;
	while (h-- > 0) {
		w = pbm->width;
		while (w-- > 0)
			*pv++ = *pi++;
		pv += (SCREEN_WIDTH - pbm->width);
	}
}

#define KEY_REPEAT_TIME	500
#define KEYTIMER_THRESH	30

/*
 *	GetKey
 */
int GetKey()
{
	u16		key;
	static u16	oldkey;
	static u32	oldtimer;

	key = READ_REG16(KEY_STS) ^ KEY_ALL;
	if (!key) {
		oldtimer = 0;
		oldkey = 0;
		return 0;
	}

	if (key == oldkey) {
		if (!oldtimer)
			oldtimer = ClockTick;
		if (ClockTick - oldtimer < KEY_REPEAT_TIME)
			return 0;
		return KEY_REPEAT | key;
	}

	if (ClockTick < oldtimer + KEYTIMER_THRESH)
		return 0;

	oldtimer = 0;
	oldkey = key;
	return key;
}

/*
 *	PutStr
 */
int PutStr(int x, int y, u8 *s)
{
	u8		c;
	int		x0;

	x0 = x;
	while (*s)
		x += XfontPutChar(x, y, &s);

	return x - x0;
}

/*
 *	Printf
 */
int Printf(int x, int y, u8 *fmt, ...)
{
	va_list		vp;
	int		n;
	u8		buf[128];

	va_start(vp, fmt);
	_vsprintf(buf, fmt, vp);
	n = PutStr(x, y, buf);
	va_end(vp);

	return n;
}

/*
 *	LeftPrintf
 */
int LeftPrintf(int x, int y, int w, u8 *fmt, ...)
{
	va_list		vp;
	int		n;
	int		thresh;
	u8		c;
	u8		*bp;
	u8		*bp1;
	u8		buf[256];

	va_start(vp, fmt);
	_vsprintf(buf, fmt, vp);

	n = 0;
	thresh = XfontGetMaxCharWidth('.') * 3;
	bp = buf;
	bp1 = NULL;
	while (*bp) {
		n += XfontGetCharWidth(&bp);
		if ((n + thresh) >= w && !bp1)
			bp1 = bp;
	}
	if (n > w && bp1)
		strcpy(bp1, "...");

	n = PutStr(x, y, buf);
	if (n < w)
		EraseScreen(x + n, y, w - n, XfontGetCharHeight());
	va_end(vp);

	return x;
}

/*
 *	RightPrintf
 */
int RightPrintf(int x, int y, int w, u8 *fmt, ...)
{
	va_list		vp;
	int		x0;
	u8		c;
	u8		*bp;
	u8		buf[256];

	va_start(vp, fmt);
	_vsprintf(buf, fmt, vp);

	x0 = x + w;
	bp = buf;
	while (*bp && x0 > x)
		x0 -= XfontGetCharWidth(&bp);
	*bp = '\0';

	if (x0 > x)
		EraseScreen(x, y, x0 - x, XfontGetCharHeight());
	PutStr(x0, y, buf);
	va_end(vp);

	return x;
}

